==> vdk-control-cli-1.3.944393829.tar.gz: vdk-control-cli-1.3.944393829/src/vdk/internal/control/command_groups/job/download_job.py <==
# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import logging
import os

import click
import click_spinner
from taurus_datajob_api import ApiResponse
from vdk.internal.control.configuration.defaults_config import load_default_team_name
from vdk.internal.control.exception.vdk_exception import VDKException
from vdk.internal.control.job.job_archive import JobArchive
from vdk.internal.control.rest_lib.factory import ApiClientFactory
from vdk.internal.control.rest_lib.rest_client_errors import ApiClientErrorDecorator
from vdk.internal.control.utils import cli_utils

log = logging.getLogger(__name__)


class JobDownloadSource:
    def __init__(self, rest_api_url: str):
        self.sources_api = ApiClientFactory(rest_api_url).get_jobs_sources_api()
        self.__job_archive = JobArchive()

    @ApiClientErrorDecorator()
    def download(self, team: str, name: str, path: str):
        log.debug(f"Download job {name} of team {team} into parent {path}")
        self.__validate_job_path(path, name)
        job_archive_path = os.path.join(path, f"{name}.zip")
        try:
            log.info(f"Downloading data job {name} in {path}/{name} ...")
            with click_spinner.spinner():
                response = self.sources_api.data_job_sources_download_with_http_info(
                    team_name=team, job_name=name, _preload_content=False
                )
                self.__write_response_to_archive(job_archive_path, response)
                self.__job_archive.unarchive_data_job(
                    job_name=name, job_archive_path=job_archive_path, job_directory=path
                )

            log.info(f"Downloaded Data Job in {path}/{name}")
        finally:
            self.__cleanup_archive(job_archive_path)

    @staticmethod
    def __write_response_to_archive(job_archive_path: str, response: ApiResponse):
        log.debug(f"Write data job source to {job_archive_path}")
        with open(job_archive_path, "wb") as w:
            w.write(response.raw_data)

    @staticmethod
    def __validate_job_path(path: str, name: str):
        job_path = os.path.join(path, name)
        if os.path.exists(job_path):
            raise VDKException(
                what=f"Cannot download data job source at given job path: {path}",
                why=f"Directory with name {name} already exists.",
                consequence="Cannot download the job and will abort.",
                countermeasure=f"Delete or move directory {job_path} or change --path location",
            )

    @staticmethod
    def __cleanup_archive(archive_path: str):
        if os.path.exists(archive_path):
            log.debug(f"Cleaning up archive, {archive_path}")
            try:
                os.remove(archive_path)
            except OSError as e:
                log.warning(
                    VDKException(
                        what=f"Cannot cleanup archive: {archive_path} as part of deployment.",
                        why=f"VDK CLI did not clean up after deploying: {e}",
                        consequence="There is a leftover archive file next to the folder containing the data job",
                        countermeasure="Either clean up the archive file "
                        "manually, or leave it as is.",
                    ).message
                )


# Below is the definition of the CLI API/UX users will be interacting
# Above is the actual implementation of the operations


@click.command(
    name="download-job",
    help="""Download a Data Job's source directory. It's a directory with the data job content inside.

Examples:

\b
# Download the source directory of a Data job 'example-job' which belongs to  'example-team'
# the source directory will be downloaded to the current working directory
vdk download-job --name example-job --team example-team

\b
# Download the source directory of a Data job 'example-job' which belongs to 'example-team'
#  /Documents/example-jobs is the parent directory where the source directory will be downloaded
vdk download-job --name example-job --team example-team --path /Documents/example-teams
    """,
)
@click.option(
    "-n",
    "--name",
    type=click.STRING,
    required=True,
    prompt="Job Name",
    help="The job name.",
)
@click.option(
    "-t",
    "--team",
    type=click.STRING,
    default=load_default_team_name(),
    required=True,
    prompt="Job Team",
    help="The team name to which the job belong to.",
)
@click.option(
    "-p",
    "--path",
    type=click.Path(exists=True, resolve_path=True),
    required=False,
    prompt="Path to the data job directory",
    default=".",
    show_default=True,
    help="The path to the parent directory where the Data Job will be downloaded."
    " If none specified it will use the current working directory.",
)
@cli_utils.rest_api_url_option()
@cli_utils.check_required_parameters
def download_job(name: str, team: str, path: str, rest_api_url: str):
    cmd = JobDownloadSource(rest_api_url)
    cmd.download(team, name, path)


# TODO: download all jobs for a team
# TODO: download old version of a job? list all versions ?

==> vdk-control-cli-1.3.1117106262.tar.gz: vdk-control-cli-1.3.1117106262/src/vdk/internal/control/command_groups/job/download_job.py <==
# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import logging
import os

import click
import click_spinner
from taurus_datajob_api import ApiResponse
from vdk.internal.control.configuration.defaults_config import load_default_team_name
from vdk.internal.control.exception.vdk_exception import VDKException
from vdk.internal.control.job.job_archive import JobArchive
from vdk.internal.control.rest_lib.factory import ApiClientFactory
from vdk.internal.control.rest_lib.rest_client_errors import ApiClientErrorDecorator
from vdk.internal.control.utils import cli_utils

log = logging.getLogger(__name__)


class JobDownloadSource:
    def __init__(self, rest_api_url: str):
        self.api = ApiClientFactory(rest_api_url)
        self.__job_archive = JobArchive()

    @ApiClientErrorDecorator()
    def download(self, team: str, name: str, path: str):
        log.debug(f"Download job {name} of team {team} into parent {path}")
        self.__validate_job_path(path, name)
        path_for_keytab = path
        job_archive_path = os.path.join(path, f"{name}.zip")
        try:
            log.info(f"Downloading data job {name} in {path}/{name} ...")
            with click_spinner.spinner():
                response = self.api.get_jobs_sources_api().data_job_sources_download_with_http_info(
                    team_name=team, job_name=name, _preload_content=False
                )
                self.__write_response_to_archive(job_archive_path, response)
                self.__job_archive.unarchive_data_job(
                    job_name=name, job_archive_path=job_archive_path, job_directory=path
                )

            log.info(f"Downloaded Data Job in {path}/{name}")
            try:
                self.__download_keytab(team, name, path_for_keytab)
            except Exception as e:
                log.warning(f"Failed to download keytab for job {name}. Error: {e}.")
        finally:
            self.__cleanup_archive(job_archive_path)

    def __download_keytab(self, team: str, name: str, path: str):
        log.info("Downloading keytab...")
        keytab_file_path = os.path.join(path, f"{name}.keytab")
        response = self.api.get_jobs_api().data_job_keytab_download_with_http_info(
            team_name=team, job_name=name, _preload_content=False
        )
        with open(keytab_file_path, "wb") as w:
            w.write(response.raw_data)
        log.info(f"Saved keytab in {keytab_file_path}")

    @staticmethod
    def __write_response_to_archive(job_archive_path: str, response: ApiResponse):
        log.debug(f"Write data job source to {job_archive_path}")
        with open(job_archive_path, "wb") as w:
            w.write(response.raw_data)

    @staticmethod
    def __validate_job_path(path: str, name: str):
        job_path = os.path.join(path, name)
        if os.path.exists(job_path):
            raise VDKException(
                what=f"Cannot download data job source at given job path: {path}",
                why=f"Directory with name {name} already exists.",
                consequence="Cannot download the job and will abort.",
                countermeasure=f"Delete or move directory {job_path} or change --path location",
            )

    @staticmethod
    def __cleanup_archive(archive_path: str):
        if os.path.exists(archive_path):
            log.debug(f"Cleaning up archive, {archive_path}")
            try:
                os.remove(archive_path)
            except OSError as e:
                log.warning(
                    VDKException(
                        what=f"Cannot cleanup archive: {archive_path} as part of deployment.",
                        why=f"VDK CLI did not clean up after deploying: {e}",
                        consequence="There is a leftover archive file next to the folder containing the data job",
                        countermeasure="Either clean up the archive file "
                        "manually, or leave it as is.",
                    ).message
                )


# Below is the definition of the CLI API/UX users will be interacting
# Above is the actual implementation of the operations


@click.command(
    name="download-job",
    help="""Download a Data Job's source directory. It's a directory with the data job content inside.

Examples:

\b
# Download the source directory of a Data job 'example-job' which belongs to  'example-team'
# the source directory will be downloaded to the current working directory
vdk download-job --name example-job --team example-team

\b
# Download the source directory of a Data job 'example-job' which belongs to 'example-team'
#  /Documents/example-jobs is the parent directory where the source directory will be downloaded
vdk download-job --name example-job --team example-team --path /Documents/example-teams
    """,
)
@click.option(
    "-n",
    "--name",
    type=click.STRING,
    required=True,
    prompt="Job Name",
    help="The job name.",
)
@click.option(
    "-t",
    "--team",
    type=click.STRING,
    default=load_default_team_name(),
    required=True,
    prompt="Job Team",
    help="The team name to which the job belong to.",
)
@click.option(
    "-p",
    "--path",
    type=click.Path(exists=True, resolve_path=True),
    required=False,
    prompt="Path to the data job directory",
    default=".",
    show_default=True,
    help="The path to the parent directory where the Data Job will be downloaded."
    " If none specified it will use the current working directory.",
)
@cli_utils.rest_api_url_option()
@cli_utils.check_required_parameters
def download_job(name: str, team: str, path: str, rest_api_url: str):
    cmd = JobDownloadSource(rest_api_url)
    cmd.download(team, name, path)


# TODO: download all jobs for a team
# TODO: download old version of a job? list all versions ?
